@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2017
+/* Copyright (c) 1997-2018
    Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
    http://www.polymake.org
 
@@ -23,7 +23,7 @@ namespace pm { namespace perl {
 SVHolder::SVHolder()
 {
    dTHX;
-   sv=newSV_type(SVt_NULL);
+   sv = newSV_type(SVt_NULL);
 }
 
 void SVHolder::forget()
@@ -41,7 +41,7 @@ SV* SVHolder::get_temp()
 SVHolder::SVHolder(SV* sv_arg, std::true_type)
 {
    dTHX;
-   sv=newSVsv(sv_arg);
+   sv = newSVsv(sv_arg);
 }
 
 void SVHolder::set_copy(SV* sv_arg)
@@ -59,43 +59,51 @@ SV* Scalar::undef()
 SV* Scalar::const_string(const char* s, size_t l)
 {
    dTHX;
-   SV* sv=newSV_type(SVt_PV);
+   SV* sv = newSV_type(SVt_PV);
    SvFLAGS(sv) |= SVf_READONLY | SVf_POK | SVp_POK;
-   SvPV_set(sv,(char*)s);
-   SvCUR_set(sv,l);
+   SvPV_set(sv, const_cast<char*>(s));
+   SvCUR_set(sv, l);
    return sv;
 }
 
 SV* Scalar::const_string_with_int(const char* s, size_t l, int i)
 {
    dTHX;
-   SV* sv=newSV_type(SVt_PVIV);
+   SV* sv = newSV_type(SVt_PVIV);
    SvFLAGS(sv) |= SVf_READONLY | SVf_POK | SVp_POK | SVf_IOK | SVp_IOK;
-   SvPV_set(sv,(char*)s);
-   SvCUR_set(sv,l);
-   SvIV_set(sv,i);
+   SvPV_set(sv, const_cast<char*>(s));
+   SvCUR_set(sv, l);
+   SvIV_set(sv, i);
+   return sv;
+}
+
+SV* Scalar::const_int(int i)
+{
+   dTHX;
+   SV* sv=newSViv(i);
+   SvREADONLY_on(sv);
    return sv;
 }
 
 int Scalar::convert_to_int(SV* sv)
 {
    MAGIC* mg=SvMAGIC(SvRV(sv));
-   const glue::scalar_vtbl* t=(const glue::scalar_vtbl*)mg->mg_virtual;
+   const glue::scalar_vtbl* t = reinterpret_cast<const glue::scalar_vtbl*>(mg->mg_virtual);
    return (t->to_int)(mg->mg_ptr);
 }
 
 double Scalar::convert_to_float(SV* sv)
 {
    MAGIC* mg=SvMAGIC(SvRV(sv));
-   const glue::scalar_vtbl* t=(const glue::scalar_vtbl*)mg->mg_virtual;
+   const glue::scalar_vtbl* t = reinterpret_cast<const glue::scalar_vtbl*>(mg->mg_virtual);
    return (t->to_float)(mg->mg_ptr);
 }
 
 SV* ArrayHolder::init_me(int size)
 {
    dTHX;
-   AV* av=newAV();
-   if (size>0) av_extend(av, size-1);
+   AV* av = newAV();
+   if (size > 0) av_extend(av, size - 1);
    return newRV_noinc((SV*)av);
 }
 
@@ -103,17 +111,17 @@ void ArrayHolder::upgrade(int size)
 {
    dTHX;
    if (!SvROK(sv)) {
-      AV* av=newAV();
-      if (size>0) av_extend(av, size-1);
+      AV* av = newAV();
+      if (size > 0) av_extend(av, size - 1);
       (void)SvUPGRADE(sv, SVt_RV);
-      SvRV(sv)=(SV*)av;
+      SvRV(sv) = (SV*)av;
       SvROK_on(sv);
    }
 }
 
 void ArrayHolder::verify() const
 {
-   if (__builtin_expect(!(SvROK(sv) && SvTYPE(SvRV(sv))==SVt_PVAV), 0))
+   if (__builtin_expect(!(SvROK(sv) && SvTYPE(SvRV(sv)) == SVt_PVAV), 0))
       throw std::runtime_error("input argument is not an array");
 }
 
@@ -125,7 +133,7 @@ void ArrayHolder::set_contains_aliases()
 int ArrayHolder::size() const
 {
    dTHX;
-   return AvFILL((AV*)SvRV(sv))+1;
+   return AvFILL((AV*)SvRV(sv)) + 1;
 }
 
 SV* ArrayHolder::operator[] (int i) const
@@ -134,39 +142,9 @@ SV* ArrayHolder::operator[] (int i) const
    return *av_fetch((AV*)SvRV(sv), i, TRUE);
 }
 
-int ArrayHolder::dim(bool& has_sparse_representation) const
-{
-   dTHX;
-   MAGIC* mg=pm_perl_array_flags_magic(aTHX_ SvRV(sv));
-   if (mg && mg->mg_len>=0 && mg->mg_obj && SvPOKp(mg->mg_obj) && SvCUR(mg->mg_obj)==3 && !strncmp(SvPVX(mg->mg_obj), "dim", 3)) {
-      has_sparse_representation=true;
-      return mg->mg_len;
-   } else {
-      has_sparse_representation=false;
-      return -1;
-   }
-}
-
-int ArrayHolder::cols() const
-{
-   dTHX;
-   MAGIC* mg=pm_perl_array_flags_magic(aTHX_ SvRV(sv));
-   if (mg && mg->mg_len>=0 && mg->mg_obj && SvPOKp(mg->mg_obj) && SvCUR(mg->mg_obj)==4 && !strncmp(SvPVX(mg->mg_obj), "cols", 4)) {
-      return mg->mg_len;
-   } else {
-      return -1;
-   }
-}
-
 bool SVHolder::is_tuple() const
 {
-   dTHX;
-   if (SvROK(sv)) {
-      MAGIC* mg=pm_perl_array_flags_magic(aTHX_ SvRV(sv));
-      return mg && mg->mg_len<0;
-   } else {
-      return false;
-   }
+   return SvROK(sv) && SvTYPE(SvRV(sv)) == SVt_PVAV && !SvOBJECT(SvRV(sv));
 }
 
 SV* ArrayHolder::shift()
@@ -201,21 +179,6 @@ void ArrayHolder::resize(int size)
    av_fill((AV*)SvRV(sv), size-1);
 }
 
-SV* make_string_array(int size, ...)
-{
-   dTHX;
-   AV* const ary=newAV();
-   av_extend(ary, size-1);
-   va_list args;
-   va_start(args, size);
-   while (--size >= 0) {
-      const char* str=va_arg(args, const char*);
-      av_push(ary, Scalar::const_string(str, strlen(str)));
-   }
-   va_end(args);
-   return newRV_noinc((SV*)ary);
-}
-
 SV* HashHolder::init_me()
 {
    dTHX;
@@ -237,10 +200,164 @@ bool HashHolder::exists(const AnyString& key) const
 SV* HashHolder::fetch(const AnyString& key, bool create) const
 {
    dTHX;
-   SV** valp=hv_fetch((HV*)SvRV(sv), key.ptr, key.len, create);
+   SV** valp = hv_fetch((HV*)SvRV(sv), key.ptr, key.len, create);
    return valp ? *valp : &PL_sv_undef;
 }
 
+ListValueInputBase::ListValueInputBase(SV* sv)
+   : dim_sv(nullptr)
+   , i(0)
+   , cols_(-1)
+   , dim_(-1)
+   , sparse_(false)
+{
+   dTHX;
+   if (SvROK(sv)) {
+      arr_or_hash = SvRV(sv);
+      const bool is_magic = SvMAGICAL(arr_or_hash) != 0;
+      if (SvTYPE(arr_or_hash) == SVt_PVAV) {
+         AV* av = (AV*)arr_or_hash;
+         size_ = (is_magic ? av_len(av) : AvFILLp(av)) + 1;
+         // TODO: remove this when XML parser vanishes
+         if (MAGIC* mg = glue::array_flags_magic(aTHX_ arr_or_hash)) {
+            if (mg->mg_len >= 0 && mg->mg_obj && SvPOKp(mg->mg_obj)) {
+               if (SvCUR(mg->mg_obj) == 3 && !strncmp(SvPVX(mg->mg_obj), "dim", 3)) {
+                  dim_ = mg->mg_len;
+                  sparse_ = true;
+               } else if (SvCUR(mg->mg_obj) == 4 && !strncmp(SvPVX(mg->mg_obj), "cols", 4)) {
+                  cols_ = mg->mg_len;
+               }
+            }
+         } else if (!is_magic) {
+            if (size_ > 0) {
+               SV* last = AvARRAY(av)[size_ - 1];
+               if (SvOBJECT(av)) {
+                  if (SvSTASH(av) == glue::Serializer_Sparse_stash) {
+                     sparse_ = true;
+                     if (size_ % 2 == 1 && SvIOK(last)) {
+                        dim_ = SvUVX(last);
+                        --size_;
+                     }
+                  }
+               } else if (SvROK(last)) {
+                  last = SvRV(last);
+                  if (SvTYPE(last) == SVt_PVHV && !SvOBJECT(last) && !SvMAGICAL(last)) {
+                     HV* attrs = (HV*)last;
+                     SV** colsp;
+                     if (HvUSEDKEYS(attrs) == 1 && (colsp = hv_fetch(attrs, "cols", 4, FALSE))) {
+                        cols_ = SvIV(*colsp);
+                        --size_;
+                     }
+                  }
+               }
+            } else {
+               cols_ = 0;  // in case, it was supposed to be an empty matrix
+            }
+         }
+         return;
+      }
+      if (!is_magic && SvTYPE(arr_or_hash) == SVt_PVHV) {
+         HV* hv = (HV*)arr_or_hash;
+         sparse_ = true;
+         UV dim_val;
+         if ((dim_sv = hv_delete_ent(hv, glue::Serializer_Sparse_dim_key, 0, SvSHARED_HASH(glue::Serializer_Sparse_dim_key)))) {
+            SvREFCNT_inc_simple_void_NN(dim_sv);  // overcome sv_2mortal
+            if (SvIOK(dim_sv)) {
+               dim_ = SvIVX(dim_sv);
+            } else if (SvPOK(dim_sv) && SvCUR(dim_sv) > 0 && grok_number(SvPVX(dim_sv), SvCUR(dim_sv), &dim_val) == IS_NUMBER_IN_UV) {
+               dim_ = dim_val;
+            } else {
+               throw std::runtime_error("wrong " + AnyString(SvPVX(glue::Serializer_Sparse_dim_key), SvCUR(glue::Serializer_Sparse_dim_key)) + " element in sparse input");
+            }
+         }
+         size_ = HvUSEDKEYS(hv);
+         hv_iterinit(hv);
+         if (!hv_iternext(hv)) i = size_;
+         return;
+      }
+   }
+   throw std::runtime_error("invalid list input: must be an array or hash");
+}
+
+bool ListValueInputBase::is_ordered() const
+{
+   return SvTYPE(arr_or_hash) == SVt_PVAV;
+}
+
+SV* ListValueInputBase::get_first() const
+{
+   dTHX;
+   if (is_ordered()) {
+      if (!sparse_representation()) {
+         return SvMAGICAL(arr_or_hash) ? *av_fetch((AV*)arr_or_hash, 0, FALSE) : AvARRAY(arr_or_hash)[0];
+      } else if (size_ > 2) {
+         return AvARRAY(arr_or_hash)[2];
+      } else {
+         return nullptr;
+      }
+   }
+   HE* entry = HvEITER((HV*)arr_or_hash);
+   return HeVAL(entry);
+}
+
+void ListValueInputBase::finish()
+{
+   if (SvTYPE(arr_or_hash) == SVt_PVHV && dim_sv) {
+      dTHX;
+      HV* hv = (HV*)arr_or_hash;
+      hv_iterinit(hv);
+      hv_store_ent(hv, glue::Serializer_Sparse_dim_key, dim_sv, SvSHARED_HASH(glue::Serializer_Sparse_dim_key));
+      dim_sv = nullptr;
+   }
+}
+
+SV* ListValueInputBase::get_next()
+{
+   dTHX;
+   if (is_ordered()) {
+      SV* sv;
+      if (sparse_representation()) {
+         sv = AvARRAY(arr_or_hash)[i + 1];
+         i += 2;
+      } else {
+         sv = SvMAGICAL(arr_or_hash) ? *av_fetch((AV*)arr_or_hash, i++, FALSE) : AvARRAY(arr_or_hash)[i++];
+      }
+      return sv;
+   }
+   HE* entry = HvEITER((HV*)arr_or_hash);
+   if (!hv_iternext((HV*)arr_or_hash)) i = size_;
+   return HeVAL(entry);
+}
+
+int ListValueInputBase::get_index() const
+{
+   if (!sparse_representation())
+      throw std::runtime_error("dense/sparse input mismatch");
+   if (is_ordered()) {
+      SV* ix_sv = AvARRAY(arr_or_hash)[i];
+      if (SvIOK(ix_sv))
+         return SvIVX(ix_sv);
+      throw std::runtime_error("sparse input - invalid index");
+   }
+   dTHX;
+   HE* entry = HvEITER((HV*)arr_or_hash);
+   I32 klen = -1;
+   const char* key = hv_iterkey(entry, &klen);
+   UV index;
+   if (klen <= 0 || grok_number(key, klen, &index) != IS_NUMBER_IN_UV)
+      throw std::runtime_error("sparse input - invalid index");
+   return index;
+}
+
+void ListValueInputBase::retrieve_key(std::string& dst) const
+{
+   dTHX;
+   HE* entry = HvEITER((HV*)arr_or_hash);
+   I32 klen = -1;
+   const char* key = hv_iterkey(entry, &klen);
+   dst.assign(key, klen);
+}
+
 Stack::Stack()
 {
    dTHX;
@@ -254,15 +371,6 @@ Stack::Stack(SV** start)
    PL_stack_sp=start;
 }
 
-Stack::Stack(bool room_for_object, int reserve)
-{
-   dTHX;
-   pi=getTHX;
-   PmStartFuncall(reserve);
-   if (room_for_object) PUSHs(&PL_sv_undef);
-   PUTBACK;
-}
-
 Stack::Stack(int reserve)
 {
    dTHX;
@@ -287,6 +395,22 @@ void Stack::xpush(SV* x) const
    PUTBACK;
 }
 
+void Stack::extend(int n)
+{
+   dTHXa(pi);
+   dSP;
+   EXTEND(SP, n);
+   PUTBACK;
+}
+
+void Stack::push(const AnyString& s) const
+{
+   dTHXa(pi);
+   dSP;
+   mPUSHp(s.ptr, s.len);
+   PUTBACK;
+}
+
 void Stack::cancel()
 {
    dTHXa(pi);
@@ -295,7 +419,8 @@ void Stack::cancel()
 
 void ListReturn::upgrade(int size)
 {
-   dTHX; dSP;
+   dTHXa(pi);
+   dSP;
    EXTEND(SP, size);
 }
 
@@ -414,9 +539,9 @@ Value::number_flags Value::classify_number() const
    if (flags & SVf_ROK) {
       SV* const obj=SvRV(sv);
       if (SvOBJECT(obj)) {
-         if (MAGIC* mg=pm_perl_get_cpp_magic(obj)) {
+         if (MAGIC* mg=glue::get_cpp_magic(obj)) {
             const glue::base_vtbl* t=(const glue::base_vtbl*)mg->mg_virtual;
-            if ((t->flags & class_is_kind_mask) == class_is_scalar)
+            if ((t->flags & ClassFlags::kind_mask) == ClassFlags::is_scalar)
                return number_is_object;
          }
       }
@@ -476,7 +601,7 @@ std::false_type* Value::retrieve(Array& x) const
       if (SvROK(x.sv)) sv_unref_flags(x.sv, SV_IMMEDIATE_UNREF);
       sv_setsv(x.sv, sv);
       x.verify();
-   } else if (options & value_allow_undef) {
+   } else if (options * ValueFlags::allow_undef) {
       x.clear();
    } else {
       throw undefined();
@@ -518,10 +643,12 @@ Value::canned_data_t Value::get_canned_data(SV* sv_arg) noexcept
    if (SvROK(sv_arg)) {
       MAGIC* mg;
       SV* obj=SvRV(sv_arg);
-      if (SvOBJECT(obj) && (mg=pm_perl_get_cpp_magic(obj)))
-         return canned_data_t(((glue::base_vtbl*)mg->mg_virtual)->type, mg->mg_ptr);
+      if (SvOBJECT(obj) && (mg=glue::get_cpp_magic(obj)))
+         return { reinterpret_cast<glue::base_vtbl*>(mg->mg_virtual)->type,
+                  mg->mg_ptr,
+                  (mg->mg_flags & uint8_t(ValueFlags::read_only)) != 0 };
    }
-   return canned_data_t(NULL, NULL);
+   return { nullptr, nullptr, false };
 }
 
 int Value::get_canned_dim(bool tell_size_if_dense) const
@@ -529,10 +656,10 @@ int Value::get_canned_dim(bool tell_size_if_dense) const
    if (SvROK(sv)) {
       MAGIC* mg;
       SV* obj=SvRV(sv);
-      if (SvOBJECT(obj) && (mg=pm_perl_get_cpp_magic(obj))) {
+      if (SvOBJECT(obj) && (mg=glue::get_cpp_magic(obj))) {
          const glue::container_vtbl* t=(const glue::container_vtbl*)mg->mg_virtual;
-         if (((t->flags & class_is_kind_mask) == class_is_container) && t->own_dimension==1) {
-            if (tell_size_if_dense || (t->flags & class_is_sparse_container))
+         if (((t->flags & ClassFlags::kind_mask) == ClassFlags::is_container) && t->own_dimension==1) {
+            if (tell_size_if_dense || t->flags * ClassFlags::is_sparse_container)
                return (t->size)(mg->mg_ptr);
          }
       }
@@ -540,35 +667,35 @@ int Value::get_canned_dim(bool tell_size_if_dense) const
    return -1;
 }
 
-Value::NoAnchors Value::put_val(long x, int, int)
+Value::NoAnchors Value::put_val(long x, int)
 {
    dTHX;
    sv_setiv(sv, x);
    return NoAnchors();
 }
 
-Value::NoAnchors Value::put_val(unsigned long x, int, int)
+Value::NoAnchors Value::put_val(unsigned long x, int)
 {
    dTHX;
    sv_setuv(sv, x);
    return NoAnchors();
 }
 
-Value::NoAnchors Value::put_val(bool x, int, int)
+Value::NoAnchors Value::put_val(bool x, int)
 {
    dTHX;
    sv_setsv(sv, x ? &PL_sv_yes : &PL_sv_no);
    return NoAnchors();
 }
 
-Value::NoAnchors Value::put_val(double x, int, int)
+Value::NoAnchors Value::put_val(double x, int)
 {
    dTHX;
    sv_setnv(sv, x);
    return NoAnchors();
 }
 
-Value::NoAnchors Value::put_val(const undefined&, int, int)
+Value::NoAnchors Value::put_val(const undefined&, int)
 {
    dTHX;
    sv_setsv(sv, &PL_sv_undef);
@@ -601,11 +728,11 @@ Value::Anchor* finalize_primitive_ref(pTHX_ const Value& v, const char* xptr, SV
    if (take_ref) {
       MAGIC* mg=glue::upgrade_to_builtin_magic_sv(aTHX_ v.get(), descr, n_anchors);
       mg->mg_ptr=(char*)xptr;
-      mg->mg_flags |= v.get_flags() & value_read_only;
+      mg->mg_flags |= uint8_t(v.get_flags() & ValueFlags::read_only);
       return n_anchors ? glue::MagicAnchors::first(mg) : nullptr;
    } else {
       MAGIC* mg=glue::upgrade_to_builtin_magic_sv(aTHX_ v.get(), descr, 0);
-      mg->mg_flags |= value_read_only;
+      mg->mg_flags |= uint8_t(ValueFlags::read_only);
       return nullptr;
    }
 }
@@ -621,7 +748,7 @@ void Value::Anchor::store(SV* sv) noexcept
 std::pair<void*, Value::Anchor*> Value::allocate_canned(SV* descr, int n_anchors) const
 {
    dTHX;
-   MAGIC* mg=glue::allocate_canned_magic(aTHX_ sv, descr, options | value_alloc_magic, n_anchors);
+   MAGIC* mg=glue::allocate_canned_magic(aTHX_ sv, descr, options | ValueFlags::alloc_magic, n_anchors);
    mg->mg_flags |= MGf_GSKIP;    // if the following constructor call dies with an exception, destroy_canned won't try to delete the non-existent object
    return { mg->mg_ptr, n_anchors ? glue::MagicAnchors::first(mg) : nullptr };
 }
@@ -638,7 +765,7 @@ SV* Value::get_constructed_canned()
    return get_temp();
 }
 
-Value::Anchor* Value::store_canned_ref_impl(void* val, SV* descr, value_flags flags, int n_anchors) const
+Value::Anchor* Value::store_canned_ref_impl(void* val, SV* descr, ValueFlags flags, int n_anchors) const
 {
    dTHX;
    MAGIC* mg=glue::allocate_canned_magic(aTHX_ sv, descr, flags, n_anchors);
@@ -749,21 +876,8 @@ ostreambuf::int_type ostreambuf::overflow(int_type c)
    return traits_type::not_eof(c);
 }
 
-namespace {
-const std::string undefined_what("unexpected undefined value of an input property");
-}
-
-undefined::undefined() :
-   std::runtime_error(undefined_what) {}
-
-SV* complain_obsolete_wrapper(const char* file, int line, const char* expr)
-{
-   dTHX;
-   sv_setpvf(ERRSV,
-             "Obsolete automatically generated code in file \"%s\", line %d: %s\nPlease remove or edit manually.\n",
-             file, line, expr);
-   throw exception();
-}
+undefined::undefined()
+   : std::runtime_error("unexpected undefined value of an input property") {}
 
 } }
 
